# !/usr/bin/perl
#####################################################################################
#  	Author:  Russel Cornelson														#
#	Last Modified: 11-13-2015														#
#																					#
#	Description:  This Perl script,  accepts a Infor Process Automation Workunit	#
#	Log and creates a report which can be used in Analyzing the Performance of the	#
#	Process Flow																	#
#																					#
#	Written for Landmark Environment version 10.1.1.30								#
#	NOTE:  May not work on other versions of Landmark Runtime Environment			#
#																					#
#	Usage: LogReporter -f <fullpathtofile> 	[-o OutputPath] [-d]	Debug Flag		#
#####################################################################################
BEGIN {
    unshift @INC, $1 if $0 =~ /^(.*)[\/\\]/;
}

#####################################################################################
#PERL Module Includes																#
#####################################################################################
use strict;
use warnings;
use lib '.\lib';
use My::Interval qw(interval intervaldebug);
use File::Find;
use Time::Piece;
use Getopt::Std;
use File::Basename;
use List::Util;
use List::MoreUtils qw(first_index);


#####################################################################################
#VARIABLE DECLARATION																#
#####################################################################################
my $outputFile;
my $reportFile;
my $inputFile;
my $path;
my $file;
my $ext;
my $workunitnum;
my $LandmarkVersion;
my $ProcessName;
my $activityStarted = "";
my $currActStartLine;
my $endLine;
my $startTime;
my $endTime;
my $lastEndTime;
my $activityID;
my $activityName;




#####################################################################################
#HANDLE SCRIPT PARAMETERS															#
#####################################################################################
$|=1;
use vars qw/$opt_f $opt_o $opt_d/;
getopts('f:o:d');
&Usage("Missing Paramter: -f C:\\PathtoFile\\filename.ext") unless ($opt_f);
&Usage("Invalid Parameter -f:  Must be a full file path and file extension") if (basename($opt_f) eq $opt_f);
&ParseFilename; 


#####################################################################################
#Main Program Flow																	#
#####################################################################################
print localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Input file set ($inputFile)\n" if ($opt_d);
print localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Output file set to ($outputFile)\n" if ($opt_d);

open (IN, $inputFile) or die localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Can not open '$inputFile'\n\t";
open (OUT,">",$outputFile) or die localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Can not open '$outputFile'\n\t";

&PrintHeaders;

while(<IN>) {													#Loop through all lines of input file
	&GetFlowInformation($_) unless ($ProcessName);				#calls function 
	
	if ($_ =~ /Activity/ and $_ =~ /started/) {					#ENCOUNTERED A LINE WHERE ACTIVITY IS STARTING
		if ($activityStarted) {
			&EndActivity($_);
			&StartActivity($_);
		}
		else {
			&StartActivity($_);
		}
	}
	elsif ($_ =~ /Activity/ and $_ =~ /completed/) {			#ENCOUNTERED A LINE WHERE ACTIVITY IS COMPLETED
		&EndActivity($_);
	}

}
print localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Landmark Version ($LandmarkVersion)\n" if ($opt_d);
print localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Process Name ($ProcessName)\n" if ($opt_d);
print localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Workunit Number ($workunitnum)\n" if ($opt_d);

close IN;
close OUT;

&BuildReport();


#####################################################################################
#END MAIN PROGRAM FLOW																#
#####################################################################################

#####################################################################################
#Subroutine: GetTime -  Pull Time from log line										#
#####################################################################################
sub GetTime
{
	my $t = shift;
	return substr($t, -16, 12);
}

#####################################################################################
#Subroutine: GetActivty -  Activity ID and Name										#
#####################################################################################
sub GetActivity
{
	my $a = shift;
	$a =~ s/,//;
	$activityID = substr($a,index($a,"id:")+3,index($a,"started")-index($a,"id:")-4);
	$activityName = substr($a,14,index($a,"id:")-15);
	
}
	
#####################################################################################
#Subroutine: StartActivity -  Handle logging of a new activity start				#
#####################################################################################
sub StartActivity
{
	$currActStartLine = shift;
	$activityStarted = "true";
	$startTime = &GetTime($currActStartLine);
	&GetActivity($currActStartLine);
	print OUT interval($endTime,$startTime) . ",SYSTEM,NextActivity\n" if ($endTime);
}


#####################################################################################
#Subroutine: EndActivity -  Handle logging of a ending activity						#
#####################################################################################
sub EndActivity
{
	$endLine = shift;
	$activityStarted = "";
	print "test" if ($activityStarted);
	$endTime = &GetTime($endLine);
	print localtime->datetime(T => q{ }) . "\tSCRIPTLOG -  " . interval($startTime,$endTime) . "," . $activityID . "," . $activityName . "\n" if ($opt_d);
	print OUT interval($startTime,$endTime) . "," . $activityID . "," . $activityName . "\n";
}

	
#####################################################################################
#Subroutine: GetFlowInformation -  Collections WU Number, Flow Name, LmrkVersion	#
#####################################################################################
sub GetFlowInformation
{
	my $line = shift;
	if ($line =~ /Workunit/ and $line =~ /started/) {
		$workunitnum = substr($line,9,index($line,"started")-10);
	}
	elsif ($line =~ /Landmark version:/) {
		$LandmarkVersion = substr($line,19);
		chomp $LandmarkVersion;
	}
	elsif ($line =~ /Process name:/) {
		$ProcessName = substr($line,15);
		chomp $ProcessName;
	}
}



#####################################################################################
#Subroutine: Usage -  Prints correct command syntax									#
#####################################################################################
sub PrintHeaders
{
	print OUT "Time(ms),ActivityID,ActivityName\n";
}

	
#####################################################################################
#Subroutine: Usage -  Prints correct command syntax									#
#####################################################################################
sub Usage
{
	my $msg = shift;
	print "\n\n$msg\n";
	print "Examples:\n";
	print "$0 -f C:\\Folder\\filename.ext\n";
	print "$0 -f C:\\Folder\\filename.ext -o C:\\OutputFolder\\\n";
	exit 1;
}

#####################################################################################
#Subroutine: ParseFileName -  Breaks file into path,filename,extension				#
#####################################################################################
sub ParseFilename
{
	if ($opt_o) {
		($file,$path,$ext) = fileparse($opt_f, '.[^\.]*');
		$inputFile = $path . $file . $ext;
		$path = $opt_o;
		$outputFile = $path . $file . ".csv";
		$reportFile = $path . $file . "REPORT.csv";
	}
	else {
		($file,$path,$ext) = fileparse($opt_f, '.[^\.]*');
		$inputFile = $path . $file . $ext;
		$outputFile = $path . $file . ".csv";
		$reportFile = $path . $file . "REPORT.csv";
	}
}


#####################################################################################
#Subroutine: BuildReport -  Creates csv Summary Report								#
#####################################################################################
sub BuildReport
{
	my @activitylist;
	my @activitytime;
	my @activitycount;
	my $ms;
	my $ai;
	my $an;
	my $idx;
	my $a;
	my $t;
	
	open (IN, $outputFile) or die localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Can not open '$outputFile'\n\t";
	open (OUT,">",$reportFile) or die localtime->datetime(T => q{ }) . "\tSCRIPTLOG - Can not open '$reportFile'\n\t";

	while(<IN>) {
		chomp $_;
		($ms,$ai,$an) = split "," , $_;
		if (!grep{$_ eq $an} @activitylist) {
			push(@activitylist, $an) unless $an eq "ActivityName";
		}
		if ($an ne "ActivityName") {
			$idx = first_index {$_ eq $an} @activitylist;
			if ($activitytime[$idx]) {
				$activitytime[$idx] = $activitytime[$idx] + $ms;
				$activitycount[$idx] = $activitycount[$idx] + 1;
			}
			else {
				$activitytime[$idx] = $ms;
				$activitycount[$idx] = 1;
			}
		}
	}
	
	print OUT "ActName,TotalMS,Count,ms each\n";
	for (my $i=0; $i <= (scalar @activitylist)-1; $i++) {
		print OUT $activitylist[$i] . "," . $activitytime[$i] . "," . $activitycount[$i] . "," . int($activitytime[$i]/$activitycount[$i]) . "\n";
	}
	

	
	close IN;
	close OUT;
}
